import { flip, getScrollParents, shift, useFloating, } from "@floating-ui/react-dom";
import Box from "@re-flex/styled/Box";
import { css, styled, useTheme } from "@re-flex/styles";
import Transition from "@re-flex/transition-group/Transition";
import clsx from "clsx";
import React, { cloneElement, useCallback, useMemo, useState, } from "react";
import useIsomorphicEffect from "../hooks/useIsomorphicEffect";
import Portal from "../Portal";
const StyledTooltip = styled(Box, true)(({ theme }) => {
    return {
        backgroundColor: theme.palette.background.paper,
        transition: "all .2s ease-in-out",
    };
});
const TooltipContent = ({ anchor, duration = 300, children, paperProps, modifiers = [], placement, onDestroy, open, onClose, onClickAway, }) => {
    const theme = useTheme();
    const { x, y, reference, floating, strategy, update, refs } = useFloating({
        placement: placement,
        middleware: [shift(), flip(), ...modifiers],
        strategy: "fixed",
    });
    const isText = typeof children === "string";
    useIsomorphicEffect(() => {
        if (!!anchor) {
            reference(anchor);
        }
    }, [anchor]);
    useIsomorphicEffect(() => {
        if (!refs.reference.current || !refs.floating.current) {
            return;
        }
        const parents = [
            ...getScrollParents(refs.reference.current),
            ...getScrollParents(refs.floating.current),
        ];
        parents.forEach((parent) => {
            parent.addEventListener("scroll", update);
            parent.addEventListener("resize", update);
        });
        return () => {
            parents.forEach((parent) => {
                parent.removeEventListener("scroll", update);
                parent.removeEventListener("resize", update);
            });
        };
    }, [refs.reference, refs.floating, update]);
    const handleEvents = useCallback((event) => {
        const anchor = refs.floating.current;
        if (!anchor)
            return;
        if (open === true && !anchor?.contains(event.target)) {
            if (onClickAway)
                onClickAway(event);
            onClose();
        }
    }, [refs.floating, open]);
    useIsomorphicEffect(() => {
        if (open) {
            document.addEventListener("mousedown", handleEvents);
            document.addEventListener("touchmove", handleEvents);
            return () => {
                document.removeEventListener("mousedown", handleEvents);
                document.removeEventListener("touchmove", handleEvents);
            };
        }
    }, [refs.floating, open]);
    return (React.createElement(Transition, { appear: true, enter: true, in: open, duration: duration, onExited: () => {
            if (open === false)
                onDestroy();
        } }, (status) => {
        const isExit = status.appear || status.exit || status.exitDone || status.exitActive;
        const isAppear = status.appearDone || status.appearActive;
        const isEnter = status.enter || status.enterDone || status.enterActive;
        return (React.createElement("div", { ref: floating, "data-show": open === true, className: clsx(css({
                zIndex: theme.zIndex.tooltip + "",
                minWidth: (refs.reference.current?.clientWidth || 60) + "px",
                position: strategy,
                textAlign: isText ? "center" : "left",
            })), style: {
                display: status.exitDone && "none",
                top: y ?? 0,
                left: x ?? 0,
            } },
            React.createElement(StyledTooltip, { bgColor: "paper", boxShadow: 1, borderRadius: 0.75, className: clsx(paperProps?.className, css({
                    transition: "all .2s ease",
                    transform: `translateY(${-20}px)`,
                    opacity: "0",
                    willChange: "transform, opacity",
                }), {
                    [css({ transform: `translateY(${-20}px)`, opacity: "0" })]: isExit,
                    [css({ transform: `translateY(${0}px)`, opacity: "1" })]: isEnter || isAppear,
                }), ...paperProps }, children)));
    }));
};
const Tooltip = ({ children, duration = 500, disablePortal, portalContainer, keepMounted = false, modifiers = [], open, placement = "right", attachTo, popperOptions, popperRef, transition, paperProps = {}, trigger = "hover", content = "Lorem ipsum dolor sit, amet consectetur adipisicing elit.", onClickAway, }) => {
    const [visible, setVisible] = useState(null);
    const [anchor, setAnchor] = useState();
    useIsomorphicEffect(() => {
        if (!!attachTo) {
            setAnchor(anchor);
        }
    }, [attachTo]);
    const onOpenChildren = (e) => {
        setAnchor(e.currentTarget);
        setVisible(visible === true ? false : true);
    };
    const actionElement = useMemo(() => {
        if (!React.isValidElement(children) &&
            !["string", "boolean", "number"].includes(typeof children)) {
            return null;
        }
        const childElement = React.Children.only(children);
        return cloneElement(childElement, {
            onClick: onOpenChildren,
        });
    }, [children, visible]);
    return (React.createElement(React.Fragment, null,
        actionElement,
        React.createElement(Portal, { disable: disablePortal, open: keepMounted ? true : visible !== null, container: portalContainer },
            React.createElement(TooltipContent, { placement: placement, modifiers: modifiers, duration: duration, open: visible, anchor: anchor, onClickAway: onClickAway, onDestroy: () => {
                    setVisible(null);
                }, onClose: () => {
                    setVisible(false);
                }, paperProps: paperProps }, content))));
};
export default Tooltip;
